@@ -14,13 +14,27 @@
  */
 
 #define unimplemented \
-	{ log_error("Command not implemented yet."); return ECMD_FAILED;}
+	log_error("Command not implemented yet."); return ECMD_FAILED
 
 /*int e2fsadm(struct cmd_context *cmd, int argc, char **argv) unimplemented*/
-int lvmsadc(struct cmd_context *cmd, int argc, char **argv) unimplemented
-int lvmsar(struct cmd_context *cmd, int argc, char **argv) unimplemented
+int lvmsadc(struct cmd_context *cmd __attribute((unused)),
+	    int argc __attribute((unused)),
+	    char **argv __attribute((unused)))
+{
+	unimplemented;
+}
 
-int pvdata(struct cmd_context *cmd, int argc, char **argv) {
+int lvmsar(struct cmd_context *cmd __attribute((unused)),
+	   int argc __attribute((unused)),
+	   char **argv __attribute((unused)))
+{
+	unimplemented;
+}
+
+int pvdata(struct cmd_context *cmd __attribute((unused)),
+	   int argc __attribute((unused)),
+	   char **argv __attribute((unused)))
+{
 	log_error("There's no 'pvdata' command in LVM2.");
 	log_error("Use lvs, pvs, vgs instead; or use vgcfgbackup and read the text file backup.");
 	log_error("Metadata in LVM1 format can still be displayed using LVM1's pvdata command.");
